<?php

namespace Dipper\Foundation\Http;

use GuzzleHttp\Client;
use GuzzleHttp\Middleware;
use GuzzleHttp\HandlerStack;
use Illuminate\Support\Collection;
use Psr\Http\Message\ResponseInterface;

class Guzzle
{
    /**
     * Used to identify handler defined by client code
     * Maybe useful in the future.
     */
    const USER_DEFINED_HANDLER = 'userDefined';

    /**
     * Http client.
     *
     * @var HttpClient
     */
    protected $client;

    /**
     * The middlewares.
     *
     * @var array
     */
    protected $middlewares = [];

    /**
     * Guzzle client config.
     *
     * @var array
     */
    protected $config;

    /**
     * 构造函数.
     */
    public function __construct(array $config = [])
    {
        $this->config = $config;
    }

    /**
     * Set guzzle config.
     *
     * @param array $defaults
     */
    public function setConfig(array $config)
    {
        $this->config = $config;
        return $this;
    }

    /**
     * Return current guzzle config.
     *
     * @return array
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Create request and parse contents to collection.
     *
     * @param string $method
     * @param array  $args
     * @param cal  $args
     *
     * @throws GuzzleException
     * @return \Illuminate\Support\Collection
     */
    public function parse($method, array $args, $callback = null)
    {
        $contents = $this->parseJSON(call_user_func_array([$this, $method], $args));

        if ($callback instanceof \Closure) {
            return $callback($contents);
        }

        return new Collection($contents);
    }

    /**
     * GET request.
     *
     * @param string $url
     * @param array  $options
     *
     * @throws HttpException
     * @return ResponseInterface
     */
    public function get($url, array $options = [])
    {
        return $this->request($url, 'GET', ['query' => $options]);
    }

    /**
     * POST request.
     *
     * @param string       $url
     * @param array|string $options
     *
     * @throws HttpException
     * @return ResponseInterface
     */
    public function post($url, $options = [])
    {
        $key = is_array($options) ? 'form_params' : 'body';

        return $this->request($url, 'POST', [$key => $options]);
    }

    /**
     * JSON request.
     *
     * @param string       $url
     * @param string|array $options
     * @param array $queries
     * @param int          $encodeOption
     *
     * @throws HttpException
     * @return ResponseInterface
     */
    public function json($url, $options = [], $encodeOption = JSON_UNESCAPED_UNICODE, $queries = [])
    {
        is_array($options) && $options = json_encode($options, $encodeOption);

        return $this->request($url, 'POST', ['query' => $queries, 'body' => $options, 'headers' => ['content-type' => 'application/json']]);
    }

    /**
     * Upload file.
     *
     * @param string $url
     * @param array  $files
     * @param array  $form
     *
     * @throws HttpException
     * @return ResponseInterface
     */
    public function upload($url, array $files = [], array $form = [], array $queries = [])
    {
        $multipart = [];

        foreach ($files as $name => $path) {
            $multipart[] = [
                'name' => $name,
                'contents' => fopen($path, 'r'),
            ];
        }

        foreach ($form as $name => $contents) {
            $multipart[] = compact('name', 'contents');
        }

        return $this->request($url, 'POST', ['query' => $queries, 'multipart' => $multipart]);
    }

    /**
     * Return GuzzleHttp\Client instance.
     *
     * @return \GuzzleHttp\Client
     */
    public function getClient()
    {
        if (!($this->client instanceof Client)) {
            $this->client = new Client();
        }

        return $this->client;
    }

    /**
     * Set GuzzleHttp\Client.
     *
     * @param \GuzzleHttp\Client $client
     *
     * @return Http
     */
    public function setClient(Client $client)
    {
        $this->client = $client;

        return $this;
    }

    /**
     * Add a middleware.
     *
     * @param mixed $middleware
     *
     * @return $this
     */
    public function addMiddleware($middleware)
    {
        array_push($this->middlewares, $middleware);
    }

    /**
     * Return all middlewares.
     *
     * @return array
     */
    public function getMiddlewares()
    {
        return $this->middlewares;
    }

    /**
     * Make a request.
     *
     * @param string $url
     * @param string $method
     * @param array  $options
     *
     * @throws HttpException
     * @return ResponseInterface
     */
    public function request($url, $method = 'GET', $options = [])
    {
        $method = strtoupper($method);

        $options = array_merge($this->config, $options);

        $options['handler'] = $this->getHandler();

        $client = $this->getClient();

        $response = $client->request($method, $url, $options);

        $client = null;

        return $response;
    }

    /**
     * @param \Psr\Http\Message\ResponseInterface|string $body
     *
     * @throws \Illuminate\Http\Exception
     * @return mixed
     */
    public function parseJSON($body)
    {
        if ($body instanceof ResponseInterface) {
            $body = $body->getBody();
        }

        //     XXX: json maybe contains special chars.
        $body = $this->invalidJSON($body);

        if (empty($body)) {
            return false;
        }

        $contents = json_decode($body, true);

        if (JSON_ERROR_NONE !== json_last_error()) {
            throw new GuzzleException('返回结果不是正确的Json格式' . json_last_error_msg());
        }

        return $contents;
    }

    /**
     * Filter the invalid JSON string.
     *
     * @param \Psr\Http\Message\StreamInterface|string $invalidJSON
     *
     * @return string
     */
    protected function invalidJSON($invalidJSON)
    {
        return preg_replace('/[\x00-\x1F\x80-\x9F]/u', '', trim($invalidJSON));
    }

    /**
     * Build a handler.
     *
     * @return HandlerStack
     */
    protected function getHandler()
    {
        $stack = HandlerStack::create();

        foreach ($this->middlewares as $middleware) {
            $stack->push($middleware);
        }

        if (isset($this->config['handler']) && is_callable($this->config['handler'])) {
            $stack->push($this->config['handler'], self::USER_DEFINED_HANDLER);
        }

        return $stack;
    }
}
