<?php

namespace Dipper\Foundation\Repository;

use Closure;
use SuperClosure\Serializer;
use Illuminate\Support\Facades\Log;
use Illuminate\Database\Eloquent\Builder as EloquentBuilder;
use Illuminate\Contracts\Cache\Repository as CacheRepository;

/**
 * Class CacheableRepository.
 */
trait CacheableRepository
{
    /**
     * 是否关闭缓存
     *
     * @var boolean
     */
    protected $cacheSkip = false;

    /**
     * @var CacheRepository
     */
    protected $cacheRepository = null;

    /**
     * Set Cache Repository.
     *
     * @param CacheRepository $repository
     *
     * @return $this
     */
    public function setCacheRepository(CacheRepository $repository)
    {
        $this->cacheRepository = $repository;

        return $this;
    }

    /**
     * Return instance of Cache Repository.
     *
     * @return CacheRepository
     */
    public function getCacheRepository()
    {
        if (is_null($this->cacheRepository)) {
            $this->cacheRepository = app(config('dipper.repository.cache.repository', 'cache'));
        }

        return $this->cacheRepository;
    }



    /**
     * Clean cache.
     *
     * @return void
     */
    public function forgetCached()
    {
        try {
            if (CacheKeys::isTaggable($this->getCacheRepository())) {
                $this->getCacheRepository()->tags(get_called_class())->flush();
            } else {
                $cacheKeys = app(CacheKeys::class)->getKeys(get_called_class());

                if (is_array($cacheKeys)) {
                    foreach ($cacheKeys as $key) {
                        $this->getCacheRepository()->forget($key);
                    }
                }
            }
        } catch (\Exception $e) {
            Log::error($e->getMessage());
        }
    }

    /**
     * Skip Cache.
     *
     * @param bool $status
     *
     * @return $this
     */
    public function skipCache($status = true)
    {
        $this->cacheSkip = $status;

        return $this;
    }

    /**
     * Get Cache key for the method.
     *
     * @param $method
     * @param $query
     *
     * @return string
     */
    public function getCacheKey($query)
    {
        $key = vsprintf(str_replace(['%', '?'], ['%%', '%s'], $query->toSql()), $query->getBindings());

        if ($query instanceof EloquentBuilder) {
            if (!empty($eagerLoads = $query->getEagerLoads())) {
                foreach ($eagerLoads as $eagerLoad) {
                    $serializer = (new Serializer)->serialize($eagerLoad);
                    $key .= '|'.$serializer;
                }
            }
        }

        $key = sprintf('%s@%s-%s', get_called_class(), $method, base64_encode($key));

        if (!CacheKeys::isTaggable($this->getCacheRepository())) {
            app(CacheKeys::class)->putKey(get_called_class(), $key);
        }

        return $key;
    }

    /**
     * Get cache minutes.
     *
     * @return int
     */
    public function getCacheMinutes()
    {
        $cacheMinutes = isset($this->cacheMinutes) ? $this->cacheMinutes : config('dipper.repository.cache.minutes', 30);

        return $cacheMinutes;
    }

    public function rememberCache($query, Closure $callback)
    {
        if ($this->isSkippedCache()) {
            return $callback($query);
        }

        $key = $this->getCacheKey($query);

        $minutes = $this->getCacheMinutes();

        $repository = $this->getCacheRepository();

        if (CacheKeys::isTaggable($this->getCacheRepository())) {
            $repository = $repository->tags(get_called_class());
        }

        $value = $repository->remember($key, $minutes, function () use ($query, $callback) {
            return $callback($query);
        });

        return $value;
    }

    /**
     * @return bool
     */
    protected function isSkippedCache()
    {
        $skipped = isset($this->cacheSkip) ? $this->cacheSkip : false;
        $request = app('Illuminate\Http\Request');
        $skipCacheParam = config('dipper.repository.cache.params.skipCache', 'skipCache');

        if ($request->has($skipCacheParam) && $request->get($skipCacheParam)) {
            $skipped = true;
        }

        return $skipped;
    }
}
