<?php
namespace Dipper\Foundation\Repository;

use Dipper\Foundation\Support\Time;
use Illuminate\Support\Facades\Request;

/**
 * Class CriteriaRepository.
 */
trait CriteriaRepository
{
    /**
     * @var array
     */
    protected $fieldSearchable = [];

    /**
     * 获取可搜索的字段配置
     *
     * @return array
     */
    public function getFieldsSearchable()
    {
        return $this->fieldSearchable;
    }

    /**
     * 标准查找
     *
     * @param array $conditions
     * @return self
     */
    public function applyConditions(array $conditions = [])
    {
        if (empty($conditions)) {
            $conditions = array_only(Request::all(), config('dipper.repository.criteria.params'));
        }

        $fieldsSearchable = $this->getFieldsSearchable();
        $search = $conditions[config('dipper.repository.criteria.params.search', 'search')];
        $searchFields = $conditions[config('dipper.repository.criteria.params.searchFields', 'searchFields')];
        $filter = $conditions[config('dipper.repository.criteria.params.filter', 'filter')];
        $orderBy = $conditions[config('dipper.repository.criteria.params.orderBy', 'orderBy')];
        $sortedBy = $conditions[config('dipper.repository.criteria.params.sortedBy', 'sortedBy')];
        $with = $conditions[config('dipper.repository.criteria.params.with', 'with')];
        $searchJoin = $conditions[config('dipper.repository.criteria.params.searchJoin', 'searchJoin')];
        $trashed = $conditions[config('dipper.repository.criteria.params.trashed', 'trashed')];
        $sortedBy = !empty($sortedBy) ? $sortedBy : 'asc';

        if ($search && is_array($fieldsSearchable) && count($fieldsSearchable)) {
            $searchFields = is_array($searchFields) || is_null($searchFields) ? $searchFields : explode(';', $searchFields);
            $fields = $this->parserFieldsSearch($fieldsSearchable, $searchFields);
            $isFirstField = true;
            $searchData = $this->parserSearchData($search);
            $search = $this->parserSearchValue($search);
            $modelForceAndWhere = strtolower($searchJoin) === 'and';

            $this->model = $this->model->where(function ($query) use ($fields, $search, $searchData, $isFirstField, $modelForceAndWhere) {
                /* @var Builder $query */
                foreach ($fields as $field => $condition) {
                    if (is_numeric($field)) {
                        $field = $condition;
                        $condition = '=';
                    }

                    $value = null;

                    $condition = trim(strtolower($condition));

                    if (isset($searchData[$field])) {
                        $value = ($condition == 'like' || $condition == 'ilike') ? "%{$searchData[$field]}%" : $searchData[$field];
                    } else {
                        if (!is_null($search)) {
                            $value = ($condition == 'like' || $condition == 'ilike') ? "%{$search}%" : $search;
                            
                            if (in_array($field, config('dipper.repository.criteria.dates'))) {
                                try {
                                    Time::parse($value);
                                } catch (\Exception $e) {
                                    continue;
                                }
                            }
                        }
                    }

                    $relation = null;
                    if (stripos($field, '.')) {
                        $explode = explode('.', $field);
                        $field = array_pop($explode);
                        $relation = implode('.', $explode);
                    }
                    $modelTableName = $query->getModel()->getTable();

                    if ($isFirstField || $modelForceAndWhere) {
                        if (!is_null($value)) {
                            if (!is_null($relation)) {
                                if ($condition == '<>' || $condition == '!=') {
                                    $query->whereDoesntHave($relation, function ($relationQuery) use ($field, $value) {
                                        if (method_exists($relationQuery->getModel(), 'bootSoftDeletes')) {
                                            $relationQuery->withTrashed();
                                        }

                                        $relationQuery->where($field, $value);
                                    });
                                } else {
                                    $query->whereHas($relation, function ($relationQuery) use ($field, $condition, $value) {
                                        if (method_exists($relationQuery->getModel(), 'bootSoftDeletes')) {
                                            $relationQuery->withTrashed();
                                        }

                                        $relationQuery->where($field, $condition, $value);
                                    });
                                }
                            } else {
                                if ($condition == 'between') {
                                    $value = str_to_array($value);
                                    $query->whereBetween($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'not between') {
                                    $value = str_to_array($value);
                                    $query->whereNotBetween($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'in') {
                                    $value = str_to_array($value);
                                    $query->whereIn($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'not in') {
                                    $value = str_to_array($value);
                                    $query->whereNotIn($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'has') {
                                    $query->whereRaw("JSON_CONTAINS($modelTableName.$field, $value)");
                                } else {
                                    $query->where($modelTableName . '.' . $field, $condition, $value);
                                }
                            }
                            $isFirstField = false;
                        }
                    } else {
                        if (!is_null($value)) {
                            if (!is_null($relation)) {
                                $query->orWhereHas($relation, function ($relationQuery) use ($field, $condition, $value) {
                                    if (method_exists($relationQuery->getModel(), 'bootSoftDeletes')) {
                                        $relationQuery->withTrashed();
                                    }
                                    
                                    $relationQuery->where($field, $condition, $value);
                                });
                            } else {
                                if ($condition == 'between') {
                                    $value = str_to_array($value);
                                    $query->orWhereBetween($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'in') {
                                    $value = str_to_array($value);
                                    $query->orWhereIn($modelTableName . '.' . $field, $value);
                                } elseif ($condition == 'has') {
                                    $query->whereRaw("JSON_CONTAINS($modelTableName.$field, $value)");
                                } else {
                                    $query->orWhere($modelTableName . '.' . $field, $condition, $value);
                                }
                            }
                        }
                    }
                }
            });
        }

        if (isset($orderBy) && !empty($orderBy)) {
            $split = explode('|', $orderBy);
            if (count($split) > 1) {
                /*
                 * ex.
                 * products|description -> join products on current_table.product_id = products.id order by description
                 *
                 * products:custom_id|products.description -> join products on current_table.custom_id = products.id order
                 * by products.description (in case both tables have same column name)
                 */
                $table = $this->model->getModel()->getTable();
                $sortTable = $split[0];
                $sortColumn = $split[1];

                $split = explode(':', $sortTable);
                if (count($split) > 1) {
                    $sortTable = $split[0];
                    $keyName = $table . '.' . $split[1];
                } else {
                    /*
                     * If you do not define which column to use as a joining column on current table, it will
                     * use a singular of a join table appended with _id
                     *
                     * ex.
                     * products -> product_id
                     */
                    $prefix = rtrim($sortTable, 's');
                    $keyName = $table . '.' . $prefix . '_id';
                }

                $this->model = $this->model
                    ->leftJoin($sortTable, $keyName, '=', $sortTable . '.id')
                    ->orderBy($sortColumn, $sortedBy)
                    ->addSelect($table . '.*');
            } else {
                $this->model = $this->model->orderBy($orderBy, $sortedBy);
            }
        }

        if (isset($filter) && !empty($filter)) {
            if (is_string($filter)) {
                $filter = explode(';', $filter);
            }

            $this->model = $this->model->select($filter);
        }

        if ($with) {
            $with = explode(';', $with);
            $this->model = $this->model->with($with);
        }

        if ($trashed) {
            switch ($trashed) {
                case 'with':
                     $this->model = $this->model->withTrashed();
                    break;
                case 'without':
                     $this->model = $this->model->withoutTrashed();
                    break;
                case 'only':
                     $this->model = $this->model->onlyTrashed();
                    break;
                default:
                    break;
            }
        }

        return $this;
    }

    /**
     * 解析搜索数据
     *
     * @param $search
     *
     * @return array
     */
    protected function parserSearchData($search)
    {
        $searchData = [];

        if (stripos($search, ':')) {
            $fields = explode(';', $search);

            foreach ($fields as $row) {
                try {
                    list($field, $value) = explode(':', $row);
                    $searchData[$field] = $value;
                } catch (\Exception $e) {
                    //Surround offset error
                }
            }
        }

        return $searchData;
    }

    /**
     * 解析搜索值
     *
     * @param $search
     *
     * @return null
     */
    protected function parserSearchValue($search)
    {
        if (stripos($search, ';') || stripos($search, ':')) {
            $values = explode(';', $search);
            foreach ($values as $value) {
                $s = explode(':', $value);
                if (count($s) == 1) {
                    return $s[0];
                }
            }

            return null;
        }

        return $search;
    }

    /**
     * 解析搜索条件
     *
     * @param array $fields
     * @param array $searchFields
     * @return void
     */
    protected function parserFieldsSearch(array $fields = [], array $searchFields = null)
    {
        if (!is_null($searchFields) && count($searchFields)) {
            $acceptedConditions = config('dipper.repository.criteria.acceptedConditions', ['=', 'like']);
            $originalFields = $fields;
            $fields = [];

            foreach ($searchFields as $index => $field) {
                $field_parts = explode(':', $field);
                $temporaryIndex = array_search($field_parts[0], $originalFields);

                if (count($field_parts) == 2) {
                    if (in_array($field_parts[1], $acceptedConditions)) {
                        unset($originalFields[$temporaryIndex]);
                        $field = $field_parts[0];
                        $condition = $field_parts[1];
                        $originalFields[$field] = $condition;
                        $searchFields[$index] = $field;
                    }
                }
            }

            foreach ($originalFields as $field => $condition) {
                if (is_numeric($field)) {
                    $field = $condition;
                    $condition = '=';
                }
                if (in_array($field, $searchFields)) {
                    $fields[$field] = $condition;
                }
            }

            if (count($fields) == 0) {
                throw new \Exception('条件' . implode(',', $searchFields) . '不允许搜索');
            }
        }

        return $fields;
    }
}
