<?php
namespace Dipper\Foundation\Repository;

use Closure;
use Illuminate\Support\Arr;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Pagination\Paginator;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Contracts\Support\Arrayable;
use Illuminate\Database\Concerns\BuildsQueries;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Container\Container as Application;
use Dipper\Foundation\Exceptions\RepositoryException;
use Illuminate\Database\Query\Builder as QueryBuilder;
use Dipper\Foundation\Exceptions\InvalidInputException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Request as RequestFacade;
use Illuminate\Database\Eloquent\Builder as EloquentBuilder;
use Dipper\Foundation\Repository\Events\RepositoryModelCreated;
use Dipper\Foundation\Repository\Events\RepositoryModelDeleted;
use Dipper\Foundation\Repository\Events\RepositoryModelRestore;
use Dipper\Foundation\Repository\Events\RepositoryModelUpdated;

/**
 * Class Repository.
 */
abstract class Repository implements RepositoryInterface, CacheableInterface, CriteriaInterface, TransformableInterface
{
    use BuildsQueries, AggregateRepository, CacheableRepository, CriteriaRepository, TransformableRepository;

    /**
     * @var Application
     */
    protected $app;

    /**
     * @var \Illuminate\Http\Request
     */
    protected $request;

    /**
     * @var Model
     */
    protected $model;

    protected $forceNoReset = false;

    /**
     * @param Application $app
     */
    public function __construct(Application $app, Request $request)
    {
        $this->app = $app;
        $this->request = $request;
        $this->makeModel();
        $this->boot();
    }

    /**
     * Specify Model class name.
     *
     * @return string
     */
    abstract public function model();

    /**
     *
     */
    public function boot()
    {
    }

    /**
     * @throws RepositoryException
     * @return Model
     */
    public function makeModel()
    {
        $model = $this->app->make($this->model());

        if (! $model instanceof Model) {
            throw new RepositoryException("模型 {$this->model()} 必须继承 Illuminate\\Database\\Eloquent\\Model");
        }

        return $this->model = $model;
    }

    /**
     * get model
     *
     * @return Model
     */
    public function getModel()
    {
        return $this->app->make($this->model());
    }

    /**
     * current model
     */
    public function currentModel()
    {
        return $this->model;
    }

    /**
     * set model
     *
     * @param $model
     * @return self
     */
    public function setModel($model)
    {
        if ($model instanceof Model) {
            $this->model = $model->setRawAttributes($model->getOriginal());
        } else {
            $this->model = $model;
        }

        return $this;
    }

    /**
     * @throws RepositoryException
     */
    public function resetModel()
    {
        if (!$this->forceNoReset) {
            $this->makeModel();
        }
    }

    /**
     */
    public function forceNoReset()
    {
        $this->forceNoReset = true;
        return $this;
    }

    /**
     * Retrieve get data of repository.
     *
     * @param array $columns
     *
     * @return mixed
     */
    public function get($columns = ['*'])
    {
        if ($columns !== ['*']) {
            $this->model = $this->model->select($columns);
        }

        $results = $this->rememberCache($this->model, function ($model) {
            return $model->get();
        });

        $this->resetModel();

        return $this->parserResult($results);
    }

    /**
     * Retrieve all data of repository.
     *
     * @param array $columns
     *
     * @return mixed
     */
    public function all($columns = ['*'])
    {
        return $this->get($columns);
    }

    /**
     * Paginate the given query into a simple paginator.
     *
     * @param  int  $limit
     * @param  array  $columns
     * @param  string  $pageName
     * @param  int|null  $page
     * @return \Illuminate\Contracts\Pagination\LengthAwarePaginator
     */
    public function paginate($limit = null, $columns = ['*'], $pageName = 'page', $page = null)
    {
        $limit = $limit ?? RequestFacade::get('limit');
        $limit = is_null($limit) ? config('dipper.repository.pagination.limit', 10) : $limit;

        if (!$limit) {
            return $this->get($columns);
        }

        $page = $page ?: Paginator::resolveCurrentPage($pageName);

        $total = $this->getCountForPagination();

        if (!$total) {
            $results = $this->returnEmpty();
        } else {
            $this->model = $this->model->forPage($page, $limit);
            $results = $this->get($columns);
        }

        $this->resetModel();

        return $this->paginator($results, $total, $limit, $page, [
            'path' => Paginator::resolveCurrentPath(),
            'pageName' => $pageName,
        ]);
    }


    /**
     * Get a paginator only supporting simple next and previous links.
     *
     * This is more efficient on larger data-sets, etc.
     *
     * @param  int  $perPage
     * @param  array  $columns
     * @param  string  $pageName
     * @param  int|null  $page
     * @return \Illuminate\Contracts\Pagination\Paginator
     */
    public function simplePaginate($limit = 15, $columns = ['*'], $pageName = 'page', $page = null)
    {
        $limit = $limit ?? RequestFacade::get('limit');
        $limit = is_null($limit) ? config('dipper.repository.pagination.limit', 10) : $limit;

        if (!$limit) {
            return $this->get($columns);
        }

        $page = $page ?: Paginator::resolveCurrentPage($pageName);

        $this->skip(($page - 1) * $limit)->take($limit + 1);

        $this->resetModel();

        return $this->simplePaginator($this->get($columns), $limit, $page, [
            'path' => Paginator::resolveCurrentPath(),
            'pageName' => $pageName,
        ]);
    }

    /**
     * Get the count of the total records for the paginator.
     *
     * @param  array  $columns
     * @return int
     */
    public function getCountForPagination()
    {
        if ($this->model instanceof EloquentBuilder) {
            $query = $this->model->toBase();
        } else {
            $query = $this->model;
        }

        $clone = clone $this;

        $query = $query->cloneWithout(['columns', 'orders', 'limit', 'offset'])
            ->cloneWithoutBindings(['select', 'order'])
            ->selectRaw(DB::raw('count(*) as aggregate'));

        $results = $this->rememberCache($query, function ($model) {
            return $model->get();
        });

        // Once we have run the pagination count query, we will get the resulting count and
        // take into account what type of query it was. When there is a group by we will
        // just return the count of the entire results set since that will be correct.
        if (isset($query->groups)) {
            return count($results);
        } elseif (! isset($results[0])) {
            return 0;
        } elseif (is_object($results[0])) {
            return (int) $results[0]->aggregate;
        }

        return (int) array_change_key_case((array) $results[0])['aggregate'];
    }

    /**
     * Find a model by its primary key.
     *
     * @param  mixed $id
     * @param  array $columns
     * @return \Illuminate\Database\Eloquent\Model|\Illuminate\Database\Eloquent\Collection|static[]|static|null
     */
    public function find($id, $columns = ['*'])
    {
        if (is_array($id) || $id instanceof Arrayable) {
            if (empty($ids)) {
                return $this->returnEmpty();
            }

            $method = 'whereIn';
        } else {
            $method = 'where';
        }

        if ($this->model instanceof EloquentBuilder) {
            $field = $this->getModel()->getQualifiedKeyName();
        } elseif ($this->model instanceof Model) {
            $field = $this->getModel()->getQualifiedKeyName();
        } else {
            $field = 'id';
        }

        $this->model = $this->model->{$method}($field, $id);

        return $this->first($columns);
    }

    /**
     * Retrieve first data of repository.
     *
     * @param array $columns
     *
     * @return mixed
     */
    public function first($columns = ['*'])
    {
        $this->model = $this->model->take(1);

        return $this->get($columns)->first();
    }

    /**
     * Retrieve first data of repository, or return new Model.
     *
     * @param array $attributes
     * @param array $values
     *
     * @return mixed
     */
    public function firstOrNew(array $attributes, array $values = [])
    {
        if (! is_null($instance = $this->where($attributes)->first())) {
            return $instance;
        }

        $model = $this->newModelInstance($attributes + $values);

        return $this->parserResult($model);
    }

    /**
     * Retrieve first data of repository, or create new Model.
     *
     * @param array $attributes
     *
     * @return mixed
     */
    public function firstOrCreate(array $attributes, array $values = [])
    {
        if (! is_null($instance = $this->where($attributes)->first())) {
            return $instance;
        }

        $model = $this->newModelInstance($attributes + $values);

        $this->setModel($model)->save();

        return $this->parserResult($model);
    }

    /**
     * Find a model by its primary key or throw an exception.
     *
     * @param  mixed $id
     * @param  array $columns
     * @return \Illuminate\Database\Eloquent\Model|\Illuminate\Database\Eloquent\Collection
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException
     */
    public function firstOrFail($columns = ['*'])
    {
        if (! is_null($model = $this->first($columns))) {
            return $model;
        }

        throw (new ModelNotFoundException)->setModel(get_class($this->model));
    }

    /**
     * Find a model by its primary key or return fresh model instance.
     *
     * @param  mixed $id
     * @param  array $columns
     * @return \Illuminate\Database\Eloquent\Model
     */
    public function findOrNew($id, $columns = ['*'])
    {
        if (! is_null($model = $this->find($id, $columns))) {
            return $model;
        }

        return $this->parserResult($this->newModelInstance());
    }

    /**
     * Execute the query and get the first result or call a callback.
     *
     * @param  \Closure|array  $columns
     * @param  \Closure|null  $callback
     * @return \Illuminate\Database\Eloquent\Model|static|mixed
     */
    public function firstOr($columns = ['*'], Closure $callback = null)
    {
        if ($columns instanceof Closure) {
            $callback = $columns;

            $columns = ['*'];
        }

        if (! is_null($model = $this->first($columns))) {
            return $model;
        }

        return call_user_func($callback);
    }

    /**
     * Create or update a record matching the attributes, and fill it with values.
     *
     * @param  array $attributes
     * @param  array $values
     * @return \Illuminate\Database\Eloquent\Model
     */
    public function updateOrCreate(array $attributes, array $values = [])
    {
        $model = $this->model->updateOrCreate($attributes, $values);

        $this->resetModel();

        event(new RepositoryModelUpdated($this, $model));

        return $this->parserResult($model);
    }

    /**
     * Increment a column's value by a given amount.
     *
     * @param  string $column
     * @param  int $amount
     * @param  array $extra
     * @return int
     */
    public function increment($column, $amount = 1, array $extra = [])
    {
        $this->model->increment($column, $amount, $extra);

        $this->resetModel();

        event(new RepositoryModelUpdated($this, $this->model));

        return $this->parserResult($this->model);
    }

    /**
     * Decrement a column's value by a given amount.
     *
     * @param  string $column
     * @param  int $amount
     * @param  array $extra
     * @return int
     */
    public function decrement($column, $amount = 1, array $extra = [])
    {
        $this->model->decrement($column, $amount, $extra);

        $this->resetModel();

        event(new RepositoryModelUpdated($this, $this->model));

        return $this->parserResult($this->model);
    }

    /**
     * Save a new model and return the instance.
     *
     * @param  array $attributes
     * @param  Model $parent
     * @return \Illuminate\Database\Eloquent\Model|$this
     */
    public function create(array $attributes = [], Model $parent = null)
    {
        $model = $this->model->create($attributes, $parent);

        $this->resetModel();

        event(new RepositoryModelCreated($this, $model));

        return $this->parserResult($model);
    }

    /**
     * insert models and return the instance.
     *
     * @param  array $attributes
     * @return \Illuminate\Database\Eloquent\Model|$this
     */
    public function insert(array $attributes = [])
    {
        $model = $this->model;

        $result = $this->model->insert($attributes);

        $this->resetModel();

        event(new RepositoryModelCreated($this, $model));

        return $result;
    }

    /**
     * Update a record in the database.
     *
     * @param  array $values
     * @param  $id
     * @return int
     */
    public function update(array $values, $id = null)
    {
        if ($id) {
            $model = $this->model->findOrFail($id);
            $model->fill($values);
            $model->save();
        } else {
            $model = $this->model->update($values);
        }

        $this->resetModel();
        event(new RepositoryModelUpdated($this, $this->getModel()));

        return $model;
    }

    /**
     * save a record in the database.
     *
     * @return int
     */
    public function save()
    {
        $this->model->save();

        $this->resetModel();

        event(new RepositoryModelUpdated($this, $this->getModel()));

        return $this->model;
    }

    /**
     * Delete a record from the database.
     *
     * @return mixed
     */
    public function delete()
    {
        if (!$this->model instanceof Model && !$this->model instanceof EloquentBuilder) {
            throw new InvalidInputException('please set up the model first.');
        }

        $deleted = $this->model->delete();

        event(new RepositoryModelDeleted($this, $this->getModel()));

        $this->resetModel();

        return $deleted;
    }

    /**
     * Destroy the models for the given IDs.
     *
     * @param  array|int $ids
     * @return int
     */
    public function destroy($ids)
    {
        $ids = Arr::wrap($ids);

        if ($this->model instanceof Model) {
            $deleted = $this->model->destroy($ids);
        } else {
            $this->model->whereIn($this->getModel()->getKeyName(), $ids)->delete();
        }


        event(new RepositoryModelDeleted($this, $this->getModel()));

        $this->resetModel();

        return $deleted;
    }

    /**
     * Restore a soft-deleted model instance.
     *
     * @return bool|null
     */
    public function restore()
    {
        $deleted = $this->model->restore();

        event(new RepositoryModelRestore($this, $this->getModel()));

        $this->resetModel();

        return $deleted;
    }

    /**
     * Sync relations.
     *
     * @param $id
     * @param $relation
     * @param $attributes
     * @param bool $detaching
     * @return mixed
     */
    public function sync($id, $relation, $attributes, $detaching = true)
    {
        return $this->find($id)->{$relation}()->sync($attributes, $detaching);
    }

    /**
     * SyncWithoutDetaching.
     *
     * @param $id
     * @param $relation
     * @param $attributes
     * @return mixed
     */
    public function syncWithoutDetaching($id, $relation, $attributes)
    {
        return $this->sync($id, $relation, $attributes, false);
    }

    /**
     * Add a generic "order by" clause if the query doesn't already have one.
     *
     * @return void
     */
    public function enforceOrderBy()
    {
        if ($this->model instanceof EloquentBuilder) {
            if (empty($this->model->getQuery()->orders) && empty($this->model->getQuery()->unionOrders)) {
                $this->orderBy($this->getModel()->getQualifiedKeyName(), 'asc');
            }
        }
    }

    /**
     * 返回空值
     *
     * @return void
     */
    public function returnEmpty()
    {
        if ($this->model instanceof Model) {
            $results = $this->model->newCollection();
        } elseif ($this->model instanceof EloquentBuilder) {
            $results = $this->getModel()->newCollection();
        } else {
            $results = collect();
        }

        return $results;
    }

    public function offsetExists($offset)
    {
        return $this->model->$offset;
    }

    public function offsetGet($offset)
    {
        return $this->model->$offset;
    }

    public function offsetSet($offset, $value)
    {
        $this->model->$offset = $value;
    }

    public function offsetUnset($offset)
    {
        unset($this->model->$offset);
    }

    public function __get($key)
    {
        return $this->model->getAttribute($key);
    }

    public function __set($name, $value)
    {
        return $this->model->setAttribute($name, $value);
    }

    public static function __callStatic($method, $parameters)
    {
        return app(static::class)->$method(...$parameters);
    }

    public function __toString()
    {
        return $this->model->toJson();
    }

    /**
     *
     * @param string $method
     * @param array $arguments
     *
     * @return mixed
     */
    public function __call($method, $arguments)
    {
        $result = $this->model->$method(...$arguments);

        if ($result instanceof EloquentBuilder || $result instanceof QueryBuilder) {
            $this->model = $result;
            return $this;
        }

        return $result;
    }
}
