<?php

use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\HtmlString;
use Dipper\Foundation\Exceptions\HttpException;

/*
|--------------------------------------------------------------------------
| 字符串处理拓展
|--------------------------------------------------------------------------
|
*/

if (!function_exists('constent_cast')) {
    /**
     * 将字符串转化为常数式 CONSTENT_CASE.
     *
     * @param  string  $value
     * @return string
     */
    function constent_cast($value)
    {
        return strtoupper(Str::snake($value));
    }
}

if (!function_exists('dot_cast')) {
    /**
     * 将字符串转化为点式 dot.cast.
     *
     * @param  string  $value
     * @return string
     */
    function dot_cast($value)
    {
        return Str::snake($value, '-');
    }
}

if (!function_exists('semicolon_cast')) {
    /**
     * 将字符串转化为分式 semicolon.cast.
     *
     * @param  string  $value
     * @return string
     */
    function semicolon_cast($value)
    {
        return Str::snake($value, ':');
    }
}

if (!function_exists('str_to_array')) {
    /**
     * Convert strings to an array.
     *
     * @param array|string $source
     * @param string $delimiter
     * @return array
     */
    function str_to_array($source, $delimiter = ',')
    {
        $array = $source;
        if (!is_array($source)) {
            $array = explode($delimiter, $source);
        }

        $append = false;
        array_map(function ($value) use (&$append) {
            if ($value === '0') {
                $append = 0;
            }
        }, $array);

        $array = array_filter($array);

        if ($append !== false) {
            array_push($array, $append);
        }

        return $array;
    }
}

if (! function_exists('base64_encode_url')) {
    /**
     * 对提供的数据进行urlsafe的base64编码
     *
     * @param $data
     * @return string
     */
    function base64_encode_url($data)
    {
        $find = ['+', '/'];
        $replace = ['-', '_'];
        return str_replace($find, $replace, base64_encode($data));
    }
}

if (! function_exists('base64_decode_url')) {
    /**
     * 对提供的urlsafe的base64编码的数据进行解码
     *
     * @param $str
     * @return bool|string
     */
    function base64_decode_url($str)
    {
        $find = ['-', '_'];
        $replace = ['+', '/'];
        return base64_decode(str_replace($find, $replace, $str));
    }
}

/*
|--------------------------------------------------------------------------
| 正则匹配判断
|--------------------------------------------------------------------------
|
*/
if (!function_exists('validate_china_phone_number')) {
    /**
     * 验证是否是中国验证码.
     *
     * @param string $number
     * @return bool
     */
    function validate_china_phone_number(string $number)
    {
        return preg_match('/^(\+?0?86\-?)?(1[3456789]\d{9})$/', $number);
    }
}

if (!function_exists('validate_username')) {
    /**
     * 验证用户名是否合法.
     *
     * @param string $username
     * @return bool
     */
    function validate_username(string $username)
    {
        return preg_match('/^[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*$/', $username);
    }
}

/*
|--------------------------------------------------------------------------
| 辅助函数
|--------------------------------------------------------------------------
|
*/

if (!function_exists('res')) {
    /**
     * API请求成功返回的Json数据.
     *
     * @param mixed $data
     * @param string $message
     * @param int $status
     * @param array $headers
     * @param int $options
     * @return json
     */
    function res($data = null, $message = '', $status = 200, $headers = [], $options = 256)
    {
        $response = [
            'code' => 0,
            'message' => $message,
            'data' => $data,
        ];

        return response()->json($response, $status, $headers, $options);
    }
}

if (!function_exists('err')) {
    /**
     * 抛出错误.
     *
     * @param string|null $message
     * @param int|null $code
     * @param int|null $statusCode
     * @param mixed $data
     * @param array $headers
     * @param int $options
     * @return void
     */
    function err($message = null, $code = -1, $data = null, $statusCode = null, $headers = [], $options = 0)
    {
        throw new HttpException($message, $code, $data, $statusCode, $headers, $options);
    }
}

if (! function_exists('referer')) {
    function referer($default = '/')
    {
        if ($referer = app('request')->get('referer')) {
        } elseif ($referer = app('request')->headers->get('referer')) {
            parse_str(parse_url($referer)['query']);
        }

        return $referer ?: $default;
    }
}

if (! function_exists('url_with_query')) {
    function url_with_query($url, array $parameters = [])
    {
        $symbol = strpos($callback, '?') ? '&' : '?';

        if (!empty($parameters)) {
            $url .= $symbol.http_build_query($parameters);
        }

        return $url;
    }
}

/*
|--------------------------------------------------------------------------
| 文件与类名
|--------------------------------------------------------------------------
|
*/

if (!function_exists('get_class_namespace_from_file')) {
    /**
     * get the class namespace form file path using token
     *
     * @param $filePathName
     *
     * @return  null|string
     */
    function get_class_namespace_from_file($filePathName)
    {
        $src = file_get_contents($filePathName);

        $tokens = token_get_all($src);
        $count = count($tokens);
        $i = 0;
        $namespace = '';
        $namespace_ok = false;
        while ($i < $count) {
            $token = $tokens[$i];
            if (is_array($token) && $token[0] === T_NAMESPACE) {
                // Found namespace declaration
                while (++$i < $count) {
                    if ($tokens[$i] === ';') {
                        $namespace_ok = true;
                        $namespace = trim($namespace);
                        break;
                    }
                    $namespace .= is_array($tokens[$i]) ? $tokens[$i][1] : $tokens[$i];
                }
                break;
            }
            $i++;
        }
        if (!$namespace_ok) {
            return null;
        } else {
            return $namespace;
        }
    }
}

if (!function_exists('get_class_name_from_file')) {
    /**
     * get the class name form file path using token
     *
     * @param $filePathName
     *
     * @return  mixed
     */
    function get_class_name_from_file($filePathName)
    {
        $php_code = file_get_contents($filePathName);

        $classes = [];
        $tokens = token_get_all($php_code);
        $count = count($tokens);
        for ($i = 2; $i < $count; $i++) {
            if ($tokens[$i - 2][0] == T_CLASS
                && $tokens[$i - 1][0] == T_WHITESPACE
                && $tokens[$i][0] == T_STRING
            ) {
                $class_name = $tokens[$i][1];
                $classes[] = $class_name;
            }
        }

        return $classes[0];
    }
}

if (!function_exists('get_class_full_name_from_file')) {
    /**
     * build and return an object of a class from its file path
     *
     * @param $filePathName
     *
     * @return  mixed
     */
    function get_class_full_name_from_file($filePathName)
    {
        return get_class_namespace_from_file($filePathName) . '\\' . get_class_name_from_file($filePathName);
    }
}

if (!function_exists('get_class_object_from_file')) {
    /**
     * build and return an object of a class from its file path
     *
     * @param $filePathName
     *
     * @return  mixed
     */
    function get_class_object_from_file($filePathName)
    {
        $classString = get_class_full_name_from_file($filePathName);

        $object = new $classString;

        return $object;
    }
}

if (!function_exists('asset')) {
    /**
     * Generate an asset path for the application.
     *
     * @param  string  $path
     * @param  bool    $secure
     * @return string
     */
    function asset($path, $secure = null)
    {
        return app('url')->asset($path, $secure);
    }
}

if (! function_exists('public_path')) {
    /**
     * Get the path to the public folder.
     *
     * @param  string  $path
     * @return string
     */
    function public_path($path = '')
    {
        return base_path().DIRECTORY_SEPARATOR.'public'.($path ? DIRECTORY_SEPARATOR.ltrim($path, DIRECTORY_SEPARATOR) : $path);
    }
}

if (! function_exists('mix')) {
    /**
     * Get the path to a versioned Mix file.
     *
     * @param  string  $path
     * @param  string  $manifestDirectory
     * @return \Illuminate\Support\HtmlString|string
     *
     * @throws \Exception
     */
    function mix($path, $manifestDirectory = '')
    {
        static $manifests = [];

        if (! Str::startsWith($path, '/')) {
            $path = "/{$path}";
        }

        if ($manifestDirectory && ! Str::startsWith($manifestDirectory, '/')) {
            $manifestDirectory = "/{$manifestDirectory}";
        }

        if (file_exists(public_path($manifestDirectory.'/hot'))) {
            $url = file_get_contents(public_path($manifestDirectory.'/hot'));

            if (Str::startsWith($url, ['http://', 'https://'])) {
                return new HtmlString(Str::after($url, ':').$path);
            }

            return new HtmlString("//localhost:8080{$path}");
        }

        $manifestPath = public_path($manifestDirectory.'/mix-manifest.json');

        if (! isset($manifests[$manifestPath])) {
            if (! file_exists($manifestPath)) {
                throw new Exception('The Mix manifest does not exist.');
            }

            $manifests[$manifestPath] = json_decode(file_get_contents($manifestPath), true);
        }

        $manifest = $manifests[$manifestPath];

        if (! isset($manifest[$path])) {
            report(new Exception("Unable to locate Mix file: {$path}."));

            if (! app('config')->get('app.debug')) {
                return $path;
            }
        }

        return new HtmlString($manifestDirectory.$manifest[$path]);
    }
}

/*
|--------------------------------------------------------------------------
| 数据库辅助
|--------------------------------------------------------------------------
|
*/

if (!function_exists('db_alter')) {
    /**
     * build and return an object of a class from its file path
     *
     * @param $filePathName
     *
     * @return  mixed
     */
    function db_alter($table, $name, $increment = 1)
    {
        if (DB::connection() instanceof \Illuminate\Database\MySqlConnection) {
            DB::statement("ALTER TABLE $table AUTO_INCREMENT = $increment, COMMENT '$name'");
        }
    }
}


/*
|--------------------------------------------------------------------------
| 数组辅助
|--------------------------------------------------------------------------
|
*/

 if (!function_exists('array_keyBy')) {
     /**
      * Key an array by a field.
      *
      * @param array $arr
      * @param string $key
      * @return array
      */
     function array_keyBy(array $arr, string $key): array
     {
         return array_combine(array_pluck($arr, $key), $arr);
     }
 }


  if (!function_exists('array_groupBy')) {
      /**
       * Group an array by a field.
       *
       * @param array $arr
       * @param string $key
       * @return array
       */
      function array_groupBy(array $arr, string $key): array
      {
          $temp = [];
          foreach ($arr as $item) {
              $temp[$item[$key]][] = $item;
          }
          return $temp;
      }
  }
