<?php

namespace Dipper\Sms;

use Dipper\Sms\Support\Config;
use Dipper\Sms\Contracts\MessageInterface;
use Dipper\Sms\Exceptions\GatewayErrorException;
use Dipper\Sms\Exceptions\NoGatewayAvailableException;

class Messenger
{
    const STATUS_SUCCESS = 'success';

    const STATUS_FAILURE = 'failure';

    /**
     * @var \Dipper\Sms\Sms
     */
    protected $sms;

    /**
     * Messenger constructor.
     *
     * @param \Dipper\Sms\Sms $sms
     */
    public function __construct(Sms $sms)
    {
        $this->sms = $sms;
    }

    /**
     * Send a message.
     *
     * @param string|array                                              $to
     * @param string|array|\Dipper\Sms\Contracts\MessageInterface $message
     * @param array                                                     $gateways
     *
     * @return array
     *
     * @throws \Dipper\Sms\Exceptions\InvalidArgumentException
     * @throws \Dipper\Sms\Exceptions\NoGatewayAvailableException
     */
    public function send($to, $message, array $gateways = [])
    {
        $message = $this->formatMessage($message);

        if (empty($gateways)) {
            $gateways = $message->getGateways();
        }

        if (empty($gateways)) {
            $gateways = $this->sms->getConfig()->get('default.gateways', []);
        }

        $gateways = $this->formatGateways($gateways);

        $strategyAppliedGateways = $this->sms->strategy()->apply($gateways);

        $results = [];
        $isSuccessful = false;
        foreach ($strategyAppliedGateways as $gateway) {
            try {
                $results[$gateway] = [
                    'status' => self::STATUS_SUCCESS,
                    'result' => $this->sms->gateway($gateway)->send($to, $message, new Config($gateways[$gateway])),
                ];

                $isSuccessful = true;

                break;
            } catch (GatewayErrorException $e) {
                $results[$gateway] = [
                    'status' => self::STATUS_FAILURE,
                    'exception' => $e,
                ];

                continue;
            }
        }

        if (!$isSuccessful) {
            throw new NoGatewayAvailableException($results);
        }

        return $results;
    }

    /**
     * @param array|string|\Dipper\Sms\Contracts\MessageInterface $message
     *
     * @return \Dipper\Sms\Contracts\MessageInterface
     */
    protected function formatMessage($message)
    {
        if (!($message instanceof MessageInterface)) {
            if (!is_array($message)) {
                $message = [
                    'content' => strval($message),
                    'template' => strval($message),
                ];
            }

            $message = new Message($message);
        }

        return $message;
    }

    /**
     * @param array $gateways
     *
     * @return array
     */
    protected function formatGateways(array $gateways)
    {
        $formatted = [];
        $config = $this->sms->getConfig();

        foreach ($gateways as $gateway => $setting) {
            if (is_int($gateway) && is_string($setting)) {
                $gateway = $setting;
                $setting = [];
            }

            $formatted[$gateway] = $setting;
            $globalSetting = $config->get("gateways.{$gateway}", []);

            if (is_string($gateway) && !empty($globalSetting) && is_array($setting)) {
                $formatted[$gateway] = array_merge($globalSetting, $setting);
            }
        }

        return $formatted;
    }
}
